//
// GNOME Dashboard
//
// MailMessageMatchRenderer.cs: Knows how to render MailMessage matches.
//
// Author:
//   Kevin Godby <godbyk@yahoo.com>
//
// FIXME: Add support for importance flag (if possible)
// FIXME: Output date in local format (internationalization)
// FIXME: Implement smarter date display (more info as date is further back in time)
// FIXME: Sort by date, newest mail first

using System;
using System.Collections;
using System.IO;
using System.Xml;

namespace Dashboard {

	class MailMessageMatchRenderer : MatchRenderer {

		public override void Startup ()
		{
			Type = "MailMessage";
		}

		public override string HTMLRenderMatches (ArrayList matches)
		{
			StringWriter sw = new StringWriter ();
			XmlWriter xw = new XmlTextWriter (sw);

			// Start the xhtml block
			xw.WriteStartElement ("div");

			// Title of block
			xw.WriteElementString ("u", "Email Messages");

			// The table of data
			xw.WriteStartElement ("table");
			xw.WriteAttributeString ("border", "0");
			xw.WriteAttributeString ("cellpadding", "0");
			xw.WriteAttributeString ("cellspacing", "0");
			xw.WriteAttributeString ("width", "100%");

			// Attempt to sort results by date (newest first)
			

			foreach (Match m in matches)
				xw.WriteRaw (HTMLRenderSingleMailMessage (m));

			xw.WriteEndElement (); // table
			xw.WriteEndElement (); // div

			// close the xhtml doc
			xw.Close ();

			return sw.ToString ();
		}

		private string HTMLRenderSingleMailMessage (Match m)
		{
			string html;

			// Make the date look pretty
			string maildate = Convert.ToString (m ["sentdate"]);
			string ParsedDate = ParseMailDate (maildate);

			// Determine which icon to display
			string mailflags = Convert.ToString (m ["flags"]);
			string Icon = ParseMailIcon (mailflags);

			// Bold new/unread message, strikeout deleted messages
			string mailfrom = Convert.ToString (m ["from"]);
			mailfrom = ParseMailStatus (mailfrom, mailflags);
			string mailsubject = Convert.ToString (m ["subject"]);
			mailsubject = ParseMailStatus (mailsubject, mailflags);
			ParsedDate = ParseMailStatus (ParsedDate, mailflags);

			// Get attachment icon (if required)
			string attachment = GetAttachmentIcon (mailflags);

			StringWriter sw = new StringWriter ();
			XmlWriter xw = new XmlTextWriter (sw);

			xw.WriteStartElement ("tr");
			xw.WriteStartElement ("td");
			xw.WriteStartElement ("img");
			xw.WriteAttributeString ("src", Icon);
			xw.WriteEndElement (); // img
			xw.WriteEndElement (); // td
			xw.WriteStartElement ("td");
			// xw.WriteAttributeString ("nowrap", "true");
			xw.WriteRaw (mailfrom);
			xw.WriteEndElement (); // td
			xw.WriteStartElement ("td");
			xw.WriteAttributeString ("colspan", "2");
			xw.WriteAttributeString ("align", "right");
			xw.WriteAttributeString ("nowrap", "1");
			xw.WriteRaw (ParsedDate);
			xw.WriteEndElement (); // td
			xw.WriteEndElement (); // tr

			xw.WriteStartElement ("tr");
			xw.WriteStartElement ("td");
			xw.WriteRaw ("&nbsp;");
			xw.WriteEndElement (); // td
			xw.WriteStartElement ("td");
			xw.WriteAttributeString ("colspan", "2");
			xw.WriteStartElement ("font");
			xw.WriteAttributeString ("color", "#666666");
			xw.WriteRaw (mailsubject);
			xw.WriteEndElement (); // font
			xw.WriteEndElement (); // td
			xw.WriteStartElement ("td");
			xw.WriteRaw (attachment);
			xw.WriteEndElement (); // td
			xw.WriteEndElement (); // tr
			
			/*
			Console.WriteLine ("---\nFrom: {0}\nDate: {1}\nSubject: {2}\nFlags: {3}\nParsed Date: {4}", 
					   m ["from"], m ["sentdate"], m ["subject"], 
					   m ["flags"], ParsedDate
					  );
			*/


			return sw.ToString ();
		}

		private string ParseMailDate (string maildate)
		{
			// I don't know if the date format returned by the EvoMail backend
			// is always the same or if it is dependent upon the locale.
			// Assuming en-US throughout.
			DateTime ParsedDate = DateTime.Parse (maildate);

			// FIXME: Not the best date format (esp. for int'l users)
			return ParsedDate.ToString ("ddd M/d");
		}

		// this might fare better in utils/evolution/camel.cs
		public enum CamelMessageFlags {
			CAMEL_MESSAGE_ANSWERED     = 1<<0,
			CAMEL_MESSAGE_DELETED      = 1<<1,
			CAMEL_MESSAGE_DRAFT        = 1<<2,
			CAMEL_MESSAGE_FLAGGED      = 1<<3,
			CAMEL_MESSAGE_SEEN         = 1<<4,
			CAMEL_MESSAGE_ATTACHMENTS  = 1<<5,
			CAMEL_MESSAGE_ANSWERED_ALL = 1<<6
		}

		private string ParseMailIcon (string mailflags)
		{
			// determine what icon to use mail-read, etc.

			string icon = "";
			int flags = int.Parse (mailflags);

			if (((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_ANSWERED) == 1) ||
			    ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_ANSWERED_ALL) == 1)) {
				icon = "internal:mail-replied.png";
			} else if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_SEEN) == 1) {
				icon = "internal:mail-read.png";
			} else {
				icon = "internal:mail-new.png";
			}

			return icon;
		}

		private string ParseMailStatus (string text, string mailflags)
		{
			StringWriter sw = new StringWriter ();
			XmlWriter xw = new XmlTextWriter (sw);

			int flags = int.Parse (mailflags);

			// make text xhtml-safe
			//xw.WriteString (text);
			//text = sw.ToString ();
			

			if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_DELETED) != 0)
				text = "<strike>" + text + "</strike>";

			if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_SEEN) == 0)
				text = "<b>" + text + "</b>";

			// FOR DEBUG PURPOSES ONLY //
			if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_DRAFT) == 1)
				Console.WriteLine ("This message is flagged as DRAFT.");

			if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_FLAGGED) == 1)
				Console.WriteLine ("This message is flagged as FLAGGED.");
			// END DEBUG //

			return text;
		}

		private string GetAttachmentIcon (string mailflags)
		{
			int flags = int.Parse (mailflags);

			if ((flags & (int) CamelMessageFlags.CAMEL_MESSAGE_ATTACHMENTS) == 0)
				return "&nbsp;";

			return "<img src=\"internal:attachment.png\" />";

		}

		private int GetDateStamp (string maildate)
		{
			DateTime xdate = DateTime.Parse (maildate);
			int DateStamp = int.Parse (maildate);
			return DateStamp;
		}

	}
}
